import asyncio
import json
from google_docs_cache_manager import GoogleDocsCacheManager
from logger import LoggerManager
from template_processor import TemplateProcessor

# Время жизни кэша по умолчанию (1 месяц)
CACHE_TIMEOUT = 60 * 60 * 24 * 30

class PromptStorage:
    def __init__(self, credentials_file, cache_directory=None, log_directory=None, max_concurrent_tasks=10, cache_ttl_seconds=CACHE_TIMEOUT, dev_mode=False):
        """
        Обертка для работы с документами Google Docs через кэш.

        :param credentials_file: Путь к файлу учетных данных Google API.
        :param cache_directory: Директория для хранения кэшированных документов.
        :param max_concurrent_tasks: Максимальное количество одновременных задач.
        :param cache_ttl_seconds: Время жизни кэша в секундах.
        :param logger: Логгер, переданный из PromptStorage. Если None, логирование отключается.
        :param dev_mode: Разрешает использование GoogleDocsProcessor (True - разрешен, False - только кеш).
        """
        self.dev_mode = dev_mode

        # Инициализируем логгер
        self.logger_manager = LoggerManager(log_dir=log_directory)
        self.logger = self.logger_manager.get_logger()
        
        if self.logger:
            self.logger.info("✅ Инициализация GoogleDocsWrapper...")

        self.cache_manager = GoogleDocsCacheManager(
            credentials_file,
            cache_directory=cache_directory,
            max_concurrent_tasks=max_concurrent_tasks,
            cache_ttl_seconds=cache_ttl_seconds,
            logger=self.logger,
            dev_mode=self.dev_mode
        )

        if self.logger:
            self.logger.info("✅ GoogleDocsWrapper успешно инициализирован.")

    async def update_documents(self, document_ids, forced_reload=False):
        if self.logger:
            self.logger.info("Запуск обновления документов: %s", document_ids)
        return await self.cache_manager.update_cache(document_ids, forced_reload=forced_reload)

    def get_cached_document(self, document_id):
        return self.cache_manager.get_cached_document(document_id)

    async def get_document_info(self, document_id, group_titles=None, tab_title=None, forced_reload=False):
        await self.update_documents([document_id], forced_reload=forced_reload)
        cached_document = self.get_cached_document(document_id)
        if not cached_document:
            return {"status": "failure", "tabs": []}
        
        tabs = cached_document.get("tabs", [])
        if group_titles:
            tabs = [tab for tab in tabs if tab.get("title") in group_titles]
        if tab_title:
            tabs = [tab for tab in tabs if tab.get("title") == tab_title]
        
        return {"tabs": tabs}  # Убрали status для совместимости с PromptStorage

    async def get_document_part(self, document_id, path="/", forced_reload=False):
        await self.update_documents([document_id], forced_reload=forced_reload)
        cached_document = self.get_cached_document(document_id)
        if not cached_document:
            return {"status": "failure", "tabs": []}
        
        tabs = cached_document.get("tabs", [])
        
        if not path or path == "/":
            return {"tabs": tabs}  # Убрали status

        path_parts = [part for part in path.split("/") if part]

        def find_tab_by_path(tabs, parts):
            if not parts:
                return None
            current_part = parts[0]
            for tab in tabs:
                if tab.get("title") == current_part:
                    if len(parts) == 1:
                        return tab
                    return find_tab_by_path(tab.get("children", []), parts[1:])
            return None

        matching_tab = find_tab_by_path(tabs, path_parts)
        if matching_tab:
            return {"tabs": [matching_tab]}  # Убрали status
        return {"tabs": []}  # Теперь всегда соответствует формату


    async def render_document_part(self, doc_id, path, parse_mode='text', template_vars=None, forced_reload=False):
        """
        Получение содержимого вкладки документа с рендерингом.

        :param doc_id: ID документа.
        :param path: Путь к вкладке.
        :param parse_mode: 'text' (по умолчанию), 'jinja2' или другой движок.
        :param template_vars: Переменные для рендеринга.
        :param forced_reload: Принудительная перезагрузка документа.
        :return: Содержимое вкладки.
        """
        def render_tabs_recursively(tabs, template_processor, template_vars):
            for tab in tabs:
                tab["tab_text"] = template_processor.render(tab["tab_text"], template_vars)
                if "children" in tab and tab["children"]:
                    render_tabs_recursively(tab["children"], template_processor, template_vars)

        found_part = await self.get_document_part(doc_id, path, forced_reload=forced_reload)

        if found_part and "tabs" in found_part:
            template_processor = TemplateProcessor(engine=parse_mode)
            render_tabs_recursively(found_part["tabs"], template_processor, template_vars)

        return found_part



import os
import logging
from logging.handlers import RotatingFileHandler
from datetime import datetime

class LoggerManager:
    def __init__(self, log_dir=None, max_log_files=3, log_level=logging.DEBUG):
        """
        Менеджер логирования для prompt-storage.
        
        :param log_dir: Директория для хранения логов. Если None или "", логирование отключается.
        :param max_log_files: Максимальное количество файлов логов.
        :param log_level: Уровень логирования (по умолчанию DEBUG).
        """
        self.log_dir = log_dir
        self.max_log_files = max_log_files
        self.log_level = log_level

        # Создаём логгер
        self.logger = logging.getLogger("prompt_storage_logger")
        self.logger.setLevel(self.log_level)

        if self.log_dir:
            os.makedirs(self.log_dir, exist_ok=True)  # Создаём папку логов
            self._cleanup_old_logs()  # Очищаем старые логи
            self._setup_handlers()  # Добавляем обработчики
        else:
            self.logger.addHandler(logging.NullHandler())  # Отключаем логирование

    def _cleanup_old_logs(self):
        """Удаляет самые старые файлы, если их количество превышает max_log_files."""
        log_files = sorted(
            [os.path.join(self.log_dir, f) for f in os.listdir(self.log_dir) if f.startswith("app_") and f.endswith(".log")],
            key=os.path.getctime
        )

        while len(log_files) > self.max_log_files:
            oldest_file = log_files.pop(0)
            os.remove(oldest_file)
            print(f"🗑 Удален старый лог-файл: {oldest_file}")

    def _setup_handlers(self):
        """Настраивает обработчики логирования."""
        log_format = "%(asctime)s - %(name)s - %(levelname)s - %(module)s.%(funcName)s - %(message)s"

        # Файл логирования с ротацией
        log_filename = os.path.join(self.log_dir, f"app_{datetime.now().strftime('%Y-%m-%d')}.log")
        file_handler = RotatingFileHandler(log_filename, maxBytes=5 * 1024 * 1024, backupCount=self.max_log_files)
        file_handler.setLevel(logging.INFO)
        file_handler.setFormatter(logging.Formatter(log_format))

        # Вывод в консоль (только WARNING и выше)
        console_handler = logging.StreamHandler()
        console_handler.setLevel(logging.WARNING)
        console_handler.setFormatter(logging.Formatter("%(name)s - %(levelname)s - %(module)s.%(funcName)s - %(message)s"))

        # Добавляем обработчики, если они ещё не добавлены
        if not self.logger.handlers:
            self.logger.addHandler(file_handler)
            self.logger.addHandler(console_handler)

    def get_logger(self):
        """Возвращает настроенный логгер."""
        return self.logger


import asyncio
import json
import os
from google.oauth2.service_account import Credentials
from googleapiclient.discovery import build
import logging
import traceback

class GoogleDocsProcessor:
    def __init__(self, credentials_file, max_concurrent_tasks=10, logger=None):
        """
        Инициализация процессора для работы с Google Docs.

        :param credentials_file: Путь к файлу с учетными данными Google API.
        :param max_concurrent_tasks: Максимальное количество одновременных задач.
        :param logger: Логгер, переданный из GoogleDocsCacheManager. Если None, логирование отключается.
        """
        self.credentials_file = credentials_file
        self.max_concurrent_tasks = max_concurrent_tasks

        # Настраиваем логгер
        self.logger = logger or logging.getLogger()
        if logger is None:
            self.logger.addHandler(logging.NullHandler())  # Отключаем логирование

        self.logger.info("✅ GoogleDocsProcessor успешно инициализирован.")

    def fetch_document_sync(self, document_id):
        """
        Синхронное получение содержимого документа Google Docs.

        :param document_id: ID документа Google Docs.
        :return: JSON-структура документа.
        """
        scopes = ['https://www.googleapis.com/auth/documents.readonly']
        creds = Credentials.from_service_account_file(self.credentials_file, scopes=scopes)
        service = build('docs', 'v1', credentials=creds)

        self.logger.debug("Загрузка документа %s", document_id)

        try:
            document = service.documents().get(documentId=document_id, includeTabsContent=True).execute()
            return {
                "document_id": document_id,
                "title": document.get("title", "Unknown title"),
                "tabs": document.get("tabs", [])  # Безопасное обращение к ключу
            }
        except Exception as e:
            self.logger.error("Ошибка при загрузке документа %s: %s", document_id, str(e))
            return {
                "document_id": document_id,
                "title": "Unknown title",
                "tabs": [],
                "status": {
                    "error_code": 2,
                    "error_message": traceback.format_exc()
                }
            }

    async def extract_tab_content(self, tab):
        """
        Извлечение текста из вкладки документа.

        :param tab: JSON-структура вкладки (tab).
        :return: Текст вкладки в виде строки.
        """
        content = tab.get('documentTab', {}).get('body', {}).get('content', []) or []
        return ''.join(
            text_element['textRun']['content']
            for element in content if 'paragraph' in element
            for text_element in element['paragraph'].get('elements', [])
            if 'textRun' in text_element
        )

    async def process_tabs(self, tabs):
        """
        Рекурсивная обработка вкладок и их преобразование в JSON.

        :param tabs: Список вкладок (основных или вложенных).
        :return: JSON-структура с данными вкладок.
        """
        async def process_single_tab(tab):
            tab_data = {
                "title": tab.get('tabProperties', {}).get('title', 'Untitled Tab'),
                "tab_id": tab.get('tabProperties', {}).get('tabId', 'Unknown'),
                "index": tab.get('tabProperties', {}).get('index', 'No Index'),
                "tab_text": await self.extract_tab_content(tab),
                "children": []
            }

            # Обработка вложенных вкладок
            if 'childTabs' in tab:
                tab_data["children"] = await self.process_tabs(tab.get('childTabs', []))

            return tab_data

        return await asyncio.gather(*(process_single_tab(tab) for tab in tabs))

    async def process_document(self, document_id):
        """
        Асинхронная обработка одного документа и сохранение результата.

        :param document_id: ID документа Google Docs.
        :return: JSON-структура обработанного документа или информация об ошибке.
        """
        try:
            self.logger.info("Начата обработка документа %s", document_id)
            document = await asyncio.to_thread(self.fetch_document_sync, document_id)

            if 'tabs' in document:
                tabs_json = await self.process_tabs(document['tabs'])
                return {
                    "document_id": document_id,
                    "title": document.get("title", "Unknown title"),
                    "tabs": tabs_json,
                    "status": {
                        "error_code": 0,  # Ошибок нет
                        "error_message": None
                    }
                }
            else:
                self.logger.warning("Документ %s не содержит вкладок", document_id)
                return {
                    "document_id": document_id,
                    "title": "Unknown title",
                    "tabs": [],
                    "status": {
                        "error_code": 1,
                        "error_message": "Документ не содержит вкладок"
                    }
                }
        except Exception as e:
            self.logger.error("Ошибка при обработке документа %s: %s", document_id, str(e))
            return {
                "document_id": document_id,
                "title": "Unknown title",
                "tabs": [],
                "status": {
                    "error_code": 2,
                    "error_message": traceback.format_exc()
                }
            }

    async def process_documents(self, document_ids):
        """
        Асинхронная обработка нескольких документов с ограничением на количество задач.

        :param document_ids: Список ID документов Google Docs.
        :return: Список загруженных документов (с данными или ошибками).
        """
        semaphore = asyncio.Semaphore(self.max_concurrent_tasks)

        async def process_with_semaphore(doc_id):
            async with semaphore:
                try:
                    return await self.process_document(doc_id)
                except Exception as e:
                    self.logger.error("Ошибка при обработке документа %s: %s", doc_id, str(e))
                    return {
                        "document_id": doc_id,
                        "title": "Unknown title",
                        "tabs": [],
                        "status": {
                            "error_code": 3,
                            "error_message": traceback.format_exc()
                        }
                    }

        self.logger.info("Начата обработка %d документов", len(document_ids))
        tasks = [process_with_semaphore(doc_id) for doc_id in document_ids]
        results = await asyncio.gather(*tasks, return_exceptions=True)

        self.logger.info("Обработка всех документов завершена")

        return results
